from PyQt5.QtWidgets import QMessageBox as QMB
from os.path import basename
import xml.etree.ElementTree as ET

def makedlg(allfound, basic, detail):
    #print(" allfound: " + str(allfound))
    #print(" basic:\n" + basic)
    #print(" detail:\n" + detail)
    msg = QMB()
    #print("    qmb")
    if allfound:
        msg.setIcon(QMB.Information)
        #print("    all: seticon")
        msg.setWindowTitle("SPARC Vocabulary Terms Found")
        #print("    all: setwindowtitle")
    else:
        msg.setIcon(QMB.Critical)
        #print("    not: seticon")
        msg.setWindowTitle("SPARC Vocabulary Terms Missing")
        #print("    not: setwindowtitle")
    msg.setText(basic)
    #print("    settext")
    if not allfound:
        msg.setDetailedText(detail)
        #print("    setdetail")
    msg.setStandardButtons(QMB.Close)
    #print("    setbuttons")
    return msg

def checkelems(root, elemname):
    empty = list()
    for elem in root.findall("./" + elemname):
        if len(elem.findall("./property/[@name=\"TraceAssociation\"]")) < 1:
            empty.append(elem.attrib["name"])
    return empty

def checkfiles(flist):
    #print(flist)
    allhavecontour = True
    allhavemarker = True
    allfound = False
    basic = str()
    detail = str()
    for fpath in flist:
        tree = ET.parse(fpath)
        root = tree.getroot()
        totalc = len(root.findall("./contour"))
        fecontours = checkelems(root, "contour")
        cfound = len(fecontours) < totalc
        totalf = len(root.findall("./marker"))
        femarkers = checkelems(root, "marker")
        mfound = len(femarkers) < totalf
        print("contours: " + str(len(fecontours)) + " of " + str(totalc))
        print("markers: " + str(len(femarkers)) + " of " + str(totalf))
        if not cfound:
            allhavecontour = False
        if not mfound:
            allhavemarker = False
        if not (cfound and mfound):
            detail += basename(fpath) + "\n"
            if len(fecontours) > 0:
                detail += " "*4 + "contours:\n"
            for contour in fecontours:
                detail += " "*8 + contour + "\n"
            if len(femarkers) > 0:
                detail += " "*4 + "markers:\n"
            for marker in femarkers:
                detail += " "*8 + marker + "\n"
            detail += "\n"
    if (not allhavecontour) and (not allhavemarker):
        #print("missing both")
        basic = "Missing SPARC Vocabulary Terms for Contours and Markers. Contact sparc@mbfbioscience.com for assistance with curating this data."
    elif not allhavecontour:
        #print("missing contour")
        basic = "Missing SPARC Vocabulary Terms for Contours. Contact sparc@mbfbioscience.com for assistance with curating this data."
    elif not allhavemarker:
        #print("missing marker")
        basic = "Missing SPARC Vocabulary Terms for Markers. Contact sparc@mbfbioscience.com for assistance with curating this data."
    else:
        allfound = True
        basic = "SPARC Vocabulary Terms found for Contours and Markers."
    return makedlg(allfound, basic, detail)

def maketacchkdlg(flist):
    return checkfiles(flist)



#used to verify that all needed inputs have been given by the user
class SetupCheckErr:
    def __init__(self):
        self.inputs = False
        self.output = False
        self.XMLChecked = False
        self.dimChecked = False
        self.contentsChecked = False
        self.scaling = False
    
    #returns true if all inputs good
    def check(self):
        if not self.inputs:
            return False
        elif not self.output:
            return False
        elif not self.XMLChecked:
            return False
        elif not self.dimChecked:
            return False
        elif not self.contentsChecked:
            return False
        elif not self.scaling:
            return False
        else:
            return True
    
    #returns string with missing inputs listed
    def getErrors(self):
        reterrs = str()
        if not self.inputs:
            reterrs += "\tInput Files\r\n"
        if not self.output:
            reterrs += "\tOutput Location\r\n"
        if not self.XMLChecked:
            reterrs += "\tXML Option\r\n"
        if not self.dimChecked:
            reterrs += "\tDimensionality\r\n"
        if not self.contentsChecked:
            reterrs += "\tContent Choice\r\n"
        if not self.scaling:
            reterrs +="\tScaling\r\n"
        else:
            reterrs += "\tNone\r\n"
        return reterrs



import sys
from PyQt5 import QtWidgets
from . ui_fijiconvertdialog import Ui_MainDialog


def main():
    app = QtWidgets.QApplication(sys.argv)
    MainDialog = QtWidgets.QDialog()
    ui = Ui_MainDialog()
    ui.setupUi(MainDialog)
    MainDialog.show()
    sys.exit(app.exec_())



from distutils.core import setup
import py2exe

setup(name="fijiconvert",
      version="0.1",
      author="Jean Fecteau",
      author_email="jfecteau@mbfbioscience.com",
      license="GNU General Public License (GPL)",
	  url="mbf.mbf",
	  windows=[{"script": "main.py"}])


from xml.etree import ElementTree
from csv import reader


class ReaderMixin:
    
    def _verifyXML(self, filename):
        if type(filename) is str and ".xml" in filename:
            return True
        else:
            return False
    
    def readXMLImage(self, filename):
        if self._verifyXML(filename):
            tree = ElementTree.parse(filename)
            root = tree.getroot()
            if root[0][0].tag == 'Image_Filename':
                return root[0][0].text
            else:
                return None
        else:
            return None
    
    def readXMLMarkers(self, filename, append = None):
        if self._verifyXML(filename):
            tree = ElementTree.parse(filename)
            root = tree.getroot()
            data = dict()
            if type(append) is dict:
                data = append
            for ind in range(1, len(root[1])):
                markerType = int(root[1][ind][0].text)
                if markerType not in data:
                    data[markerType] = list()
                for marker in range(1, len(root[1][ind])):
                    coords = (float(root[1][ind][marker][0].text), float(root[1][ind][marker][1].text), float(root[1][ind][marker][2].text))
                    data[markerType].append(coords)
            return data
        else:
            return None
    
    def readROI(self, filename):
        if type(filename) is str and ".txt" in filename:
            roi = list()
            with open(filename, newline="") as csvfile:
                rdr = reader(csvfile, delimiter="\t")
                for row in rdr:
                    coords = list()
                    for coord in row:
                        coords.append(int(coord))
                    roi.append(tuple(coords))
            return roi
        else:
            return None



class WriterMixin:
    #write a single XML string containing a point
    def xmlPointStr(self, coords):
        if len(coords) > 1:
            retstr = '<point x="'
            retstr += '{0:.2f}'.format(coords[0])
            retstr += '" y="'
            retstr += '{0:.2f}'.format(coords[1])
            retstr += '" z="'
            if len(coords) > 2:
                retstr += '{0:.2f}'.format(coords[2])
            else:
                retstr += '0.00'
            retstr += '" d="'
            if len(coords) == 5:
                retstr += '{0:.2f}'.format(coords[4])
            else:
                retstr += '1.00'
            retstr += '" sid="'
            if len(coords) > 3:
                retstr += 'S' + str(coords[3])
            else:
                retstr += 'S1'
            retstr += '"/>'
            return retstr
        else:
            return ""
            
    #write a single XML string containing a marker
    def xmlMarkerStr(self, number, markerRef):
        if (type(markerRef) is dict and
             number >= 1 and
             number < len(markerRef)):
            retstr = '<marker type="'
            retstr += markerRef[number]
            retstr += '" color="#FFFFFF" name="Marker '
            retstr += str(number)
            retstr += '" varicosity="false">'
            return retstr
        else:
            return ""
    
    #write a single XML string containing a contour
    def xmlContourStr(self, number, closed = True):
        if number >= 0:
            retstr = '<contour name="Contour Name '
            retstr += str(number)
            retstr += '" color="#FFFFFF" closed="'
            if closed:
                retstr += 'true'
            else:
                retstr += 'false'
            retstr += '" shape="Contour">'
            return retstr
        else:
            return ""
    
    def xmlSectionStr(self, number, spacing = 1, cut = None):
        if type(cut) is None:
            cut = spacing
        if number >= 1:
            retstr = '<section sid="S'
            retstr += str(number)
            retstr += '" name="Section '
            retstr += str(number)
            retstr += '" top="'
            retstr += str((number - 1) * spacing)
            retstr += '" cutthickness="'
            retstr += str(spacing)
            retstr += '" mountedthickness="'
            retstr += str(spacing)
            retstr += '"/>'
            return retstr
        else:
            return "" 
    
    def xmlMarkerBlock(self, number, points, markerRef):
        if (number > 0 and
             type(points) is list and
             len(points) > 0 and
             type(points[0]) is tuple):
            retstr = self.xmlMarkerStr(number, markerRef)
            retstr += '\r\n'
            for point in points:
                retstr += '  '
                retstr += self.xmlPointStr(point)
                retstr += '\r\n'
            retstr += '</marker>\r\n'
            return retstr
        else:
            return ""
    
    def xmlContourBlock(self, number, points, boilerplate):
        if (number > 0 and
             type(points) is list and
             type(points[0]) is tuple):
            retstr = self.xmlContourStr(number)
            retstr += '\r\n'
            retstr += boilerplate
            for point in points:
                retstr += '  '
                retstr += self.xmlPointStr(point)
                retstr += '\r\n'
            retstr += '</contour>\r\n'
            return retstr
        else:
            return ""
    
    def xmlFilefactsBlock(self, sections = 0, spacing = 1, cut = None):
        if type(cut) is None:
            cut = spacing
        if sections > 0:
            retstr = '<filefacts>\r\n'
            for section in range(1, sections + 1):
                retstr += '  '
                retstr += self.xmlSectionStr(section, spacing, cut)
                retstr += '\r\n'
            retstr += '</filefacts>\r\n'
            return retstr
        else:
            return ""
    
    def xmlImageBlock(self, filenames, scaling, spacing = 0.0):
        if (type(filenames) is list and
             type(filenames[0]) is str and
             type(scaling) is tuple):
            retstr = '<images>\r\n'
            for ind in range(len(filenames)):
                retstr += '  <image>\r\n'
                retstr += '    <filename>'
                retstr += filenames[ind]
                retstr += '</filename>\r\n'
                retstr += '    <scale x="'
                retstr += '{0:.5f}'.format(scaling[0])
                retstr += '" y="'
                retstr += '{0:.5f}'.format(scaling[1])
                retstr += '"/>\r\n'
                retstr += '    <coord x="0.00000" y="0.00000" z="'
                retstr += '{0:.5f}'.format(ind * spacing)
                retstr += '"/>\r\n'
                retstr += '  </image>\r\n'
            retstr += '</images>\r\n'
            return retstr
        else:
            return ""
    
    def writeFile(self, filename, body):
        with open(filename, mode='w', newline='') as file:
            file.write(body)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\MBFDev\CustomApplicationWork\SPARC\verify XML data\gui_output.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import Qt
from os import path
import xml.etree.ElementTree as ET
from . tacchkdlg import maketacchkdlg

class Ui_mdgForm(object):
    def __init__(self):
        self.fnametopath = dict()
        self.fnametotree = dict()
        self.justsaved = False
        self.cols = [
            "Filename",
            "Subject: Species",
            "Subject: ID",
            "Subject: Sex",
            "Subject: Age",
            "Atlas: Organ",
            "Atlas: Label",
            "Atlas: Root ID"
        ]
        
    def setupUi(self, mdgForm):
        mdgForm.setObjectName("mdgForm")
        mdgForm.resize(1200, 300)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(mdgForm)
        self.horizontalLayout_2.setContentsMargins(5, 5, 5, 5)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.vLayout = QtWidgets.QVBoxLayout()
        self.vLayout.setObjectName("vLayout")
        self.btnAdd = QtWidgets.QPushButton(mdgForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAdd.sizePolicy().hasHeightForWidth())
        self.btnAdd.setSizePolicy(sizePolicy)
        self.btnAdd.setMaximumSize(QtCore.QSize(300, 16777215))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btnAdd.setFont(font)
        self.btnAdd.setFocusPolicy(QtCore.Qt.NoFocus)
        self.btnAdd.setObjectName("btnAdd")
        self.vLayout.addWidget(self.btnAdd)
        self.btnRemove = QtWidgets.QPushButton(mdgForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnRemove.sizePolicy().hasHeightForWidth())
        self.btnRemove.setSizePolicy(sizePolicy)
        self.btnRemove.setMaximumSize(QtCore.QSize(300, 16777215))
        self.btnRemove.setFont(font)
        self.btnRemove.setFocusPolicy(QtCore.Qt.NoFocus)
        self.btnRemove.setObjectName("btnRemove")
        self.vLayout.addWidget(self.btnRemove)
        self.hLine = QtWidgets.QFrame(mdgForm)
        self.hLine.setFrameShape(QtWidgets.QFrame.HLine)
        self.hLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.hLine.setObjectName("hLine")
        self.vLayout.addWidget(self.hLine)
        self.btnSave = QtWidgets.QPushButton(mdgForm)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btnSave.setFont(font)
        self.btnSave.setObjectName("btnSave")
        self.vLayout.addWidget(self.btnSave)
        self.horizontalLayout_2.addLayout(self.vLayout)
        self.vLine = QtWidgets.QFrame(mdgForm)
        self.vLine.setFrameShape(QtWidgets.QFrame.VLine)
        self.vLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.vLine.setObjectName("vLine")
        self.horizontalLayout_2.addWidget(self.vLine)
        self.tblFiles = QtWidgets.QTableWidget(mdgForm)
        self.tblFiles.setFocusPolicy(QtCore.Qt.NoFocus)
        self.tblFiles.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tblFiles.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tblFiles.setColumnCount(8)
        self.tblFiles.setObjectName("tblFiles")
        self.tblFiles.setRowCount(0)
        self.tblFiles.setHorizontalHeaderLabels(self.cols)
        self.horizontalLayout_2.addWidget(self.tblFiles)
        self.horizontalLayout_2.setStretch(2, 1)

        self.retranslateUi(mdgForm)
        QtCore.QMetaObject.connectSlotsByName(mdgForm)

        self.makeconnections()

    def retranslateUi(self, mdgForm):
        _translate = QtCore.QCoreApplication.translate
        mdgForm.setWindowTitle(_translate("mdgForm", "Form"))
        self.btnAdd.setText(_translate("mdgForm", "Add Files"))
        self.btnRemove.setText(_translate("mdgForm", "Remove Files"))
        self.btnSave.setText(_translate("mdgForm", "Save Changes"))

    def makeconnections(self):
        self.btnAdd.released.connect(self.on_btnAdd_clicked)
        self.btnRemove.released.connect(self.on_btnRemove_clicked)
        self.btnSave.released.connect(self.on_btnSave_clicked)
        self.tblFiles.itemSelectionChanged.connect(self.on_tblFiles_item_changed)

    def checkXML(self, root):
        #print(root)
        itfound = False
        rdict = dict()
        for colind in range(1, len(self.cols)):
            rdict[self.cols[colind]] = ""
        for item in root:
            #print("  " + str(item))
            if item.tag == "sparcdata":
                itfound = True
                sdroot = item
                sdfound = False
                atfound = False
                for child in sdroot:
                    #print("    " + str(child))
                    if child.tag == "subject":
                        sdfound = True
                        keyset = set(child.attrib.keys())
                        if "species" in keyset:
                            rdict[self.cols[1]] = child.attrib["species"]
                        if "subjectid" in keyset:
                            rdict[self.cols[2]] = child.attrib["subjectid"]
                        if "sex" in keyset:
                            rdict[self.cols[3]] = child.attrib["sex"]
                        if "age" in keyset:
                            rdict[self.cols[4]] = child.attrib["age"]
                    elif child.tag == "atlas":
                        atfound = True
                        keyset = child.attrib.keys()
                        if "organ" in keyset:
                            rdict[self.cols[5]] = child.attrib["organ"]
                        if "label" in keyset:
                            rdict[self.cols[6]] = child.attrib["label"]
                        if "rootid" in keyset:
                            rdict[self.cols[7]] = child.attrib["rootid"]
                    if sdfound and atfound:
                        break
        return rdict

    def rebuildtable(self):
        if self.justsaved:
            self.tblFiles.clearContents()
            self.tblFiles.setRowCount(0)
            self.addfromlist(list(self.fnametopath.values()))
            self.justsaved = False

    def addfromlist(self, flist):
        for fpath in flist:
            #print("  got fpath")
            tree = ET.parse(fpath)
            #print("  got tree")
            root = tree.getroot()
            #print("  got root")
            rowinfo = self.checkXML(root)
            #print("  checked XML")
            fname = path.basename(fpath)
            #print("  got fname: " + fname)
            self.fnametopath[fname] = fpath
            self.fnametotree[fname] = tree
            #print("  added dict data")
            self.tblFiles.insertRow(self.tblFiles.rowCount())
            #print("  inserted row")
            newitem = QtWidgets.QTableWidgetItem(fname)
            #print("  made new item")
            newitem.setFlags(newitem.flags() ^ Qt.ItemIsEditable)
            #print("  set item flags")
            filled = not "" in rowinfo.values()
            if filled:
                newitem.setBackground(Qt.green)
            self.tblFiles.setItem(self.tblFiles.rowCount()-1, 0, newitem)
            #print("  set item")
            for colind in range(1,len(self.cols)):
                #print(rowinfo)
                #print()
                #print(colind)
                #print()
                #print(self.cols[colind])
                #print("    " + str(colind))
                text = rowinfo[self.cols[colind]]
                #print("    got text")
                newitem = QtWidgets.QTableWidgetItem(text)
                #print("    made new item")
                if filled:
                    newitem.setBackground(Qt.green)
                elif not text:
                    if colind != 2:
                        newitem.setBackground(Qt.red)
                    else:
                        newitem.setBackground(Qt.yellow)
                self.tblFiles.setItem(self.tblFiles.rowCount()-1, colind, newitem)
                #print("    set new item")
        self.tblFiles.resizeColumnsToContents()
        #print("table resized")

    def on_tblFiles_item_changed(self):
        for rowind in range(self.tblFiles.rowCount()):
            rowtxts = list()
            for colind in range(self.tblFiles.columnCount()):
                item = self.tblFiles.item(rowind, colind)
                rowtxts.append(item.text())
            if not "" in rowtxts:
                for colind in range(self.tblFiles.columnCount()):
                    self.tblFiles.item(rowind, colind).setBackground(Qt.green)
            else:
                for colind in range(self.tblFiles.columnCount()):
                    text = self.tblFiles.item(rowind, colind).text()
                    if text:
                        self.tblFiles.item(rowind, colind).setBackground(Qt.white)
                    elif colind != 2:
                        self.tblFiles.item(rowind, colind).setBackground(Qt.red)
                    else:
                        self.tblFiles.item(rowind, colind).setBackground(Qt.yellow)

    def on_btnAdd_clicked(self):
        #print("clicked")
        self.rebuildtable()
        #print("rebuilt")
        flist, _ = QtWidgets.QFileDialog.getOpenFileNames(caption="Add XML Files", directory="C:\\", filter="XML Files (*.xml)")
        #print("got flist")
        fset = set(flist)
        #print("got set")
        fset = fset - set(self.fnametopath.values())
        #print("subtracted set")
        flist = list(fset)
        #print("got list")
        self.addfromlist(flist)
        #print("added")
            
    def on_btnRemove_clicked(self):
        self.rebuildtable()
        rlist = list()
        sranges = self.tblFiles.selectedRanges()
        #print(sranges)
        for srange in sranges:
            start = srange.topRow()
            end = srange.bottomRow()
            for row in range(start, end + 1):
                rlist.append(row)
        rlist.sort(reverse=True)
        #print(rlist)
        for row in rlist:
            #print("  " + str(row))
            fitem = self.tblFiles.item(row, 0)
            fname = fitem.text()
            #print("  fname:" + fname)
            self.tblFiles.removeRow(row)
            #print("  removed")
            del self.fnametopath[fname]
            del self.fnametotree[fname]
            #print("  deleted")
        self.tblFiles.resizeColumnsToContents()

    def on_btnSave_clicked(self):
        #print("saving")
        for rowind in range(self.tblFiles.rowCount()):
            fitem = self.tblFiles.item(rowind,0)
            #print(fitem.background().color())
            fname = fitem.text()
            #print("  " + fname)
            #print(fname)
            tree = self.fnametotree[fname]
            #print("  got tree")
            #print(tree)
            root = tree.getroot()
            #print("  got root")
            #print(root)
            sparcdata = root.find("sparcdata")
            #print("  got sparcdata")
            subject = None
            atlas = None
            if sparcdata is None:
                #print("    sparcdata is None")
                sparcdata = ET.SubElement(root, "sparcdata")
                subject = ET.SubElement(sparcdata, "subject")
                subject.set("species","")
                subject.set("subjectid","")
                subject.set("sex","")
                subject.set("age","")
                atlas = ET.SubElement(sparcdata, "atlas")
                atlas.set("organ","")
                atlas.set("label","")
                atlas.set("rootid","")
            else:
                #print("    sparcdata exists")
                subject = sparcdata.find("subject")
                if subject is None:
                    #print("    subject is None")
                    subject = ET.SubElement(sparcdata, "subject")
                    subject.set("species","")
                    subject.set("subjectid","")
                    subject.set("sex","")
                    subject.set("age","")
                #else:
                    #print("    subject exists")
                atlas = sparcdata.find("atlas")
                if atlas is None:
                    #print("  atlas is None")
                    atlas = ET.SubElement(sparcdata, "atlas")
                    atlas.set("organ","")
                    atlas.set("label","")
                    atlas.set("rootid","")
                #else:
                    #print("    atlas exists")
            #print("  got all data places")
            for colind in range(1,len(self.cols)):
                ditem = self.tblFiles.item(rowind,colind)
                #print(ditem.background().color())
                reptext = ditem.text()
                #if len(reptext) < 1:
                    #print("    [reptext empty]")
                #else:
                    #print("    " + reptext)
                if colind == 1:
                    subject.attrib["species"] = reptext
                elif colind == 2:
                    subject.attrib["subjectid"] = reptext
                elif colind == 3:
                    subject.attrib["sex"] = reptext
                elif colind == 4:
                    subject.attrib["age"] = reptext
                if colind == 5:
                    atlas.attrib["organ"] = reptext
                elif colind == 6:
                    atlas.attrib["label"] = reptext
                elif colind == 7:
                    atlas.attrib["rootid"] = reptext
            #print("  finished looping")
            fpath = self.fnametopath[fname]
            #print("  fpath: " + fpath)
            tree.write(fpath)
            #print("  tree written")
            for colind in range(self.tblFiles.columnCount()):
                ritem = self.tblFiles.item(rowind,colind)
                ritem.setBackground(Qt.cyan)
        self.tblFiles.clearSelection()
        self.justsaved = True
        #print("making msgbox")
        tacmsgbox = maketacchkdlg(list(self.fnametopath.values()))
        #print("showing msgbox")
        _ = tacmsgbox.exec_()
        #print("msgbox closed")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'fijiconvertdialog.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from os import path
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5 import QtWidgets, QtCore
from PyQt5.QtWidgets import *
from . setupcheckerr import SetupCheckErr
from . converter import Converter
from . xml_gui import Ui_mdgForm

class Ui_MainDialog(object):
    def setupUi(self, MainDialog):
        MainDialog.setObjectName("MainDialog")
        MainDialog.resize(648, 450)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainDialog.sizePolicy().hasHeightForWidth())
        MainDialog.setSizePolicy(sizePolicy)
        MainDialog.setMinimumSize(QtCore.QSize(0, 450))
        MainDialog.setMaximumSize(QtCore.QSize(16777215, 450))
        self.mdlayout = QtWidgets.QVBoxLayout(MainDialog)
        self.mdlayout.setObjectName("mdlayout")
        self.MainDialogLayout = QtWidgets.QVBoxLayout()
        self.MainDialogLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.MainDialogLayout.setContentsMargins(0, -1, -1, -1)
        self.MainDialogLayout.setObjectName("MainDialogLayout")
        self.FileIOWidget = QtWidgets.QWidget(MainDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.FileIOWidget.sizePolicy().hasHeightForWidth())
        self.FileIOWidget.setSizePolicy(sizePolicy)
        self.FileIOWidget.setObjectName("FileIOWidget")
        self.fiolayout = QtWidgets.QVBoxLayout(self.FileIOWidget)
        self.fiolayout.setContentsMargins(0, 0, 0, 0)
        self.fiolayout.setObjectName("fiolayout")
        self.FileIOLayout = QtWidgets.QVBoxLayout()
        self.FileIOLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.FileIOLayout.setSpacing(1)
        self.FileIOLayout.setObjectName("FileIOLayout")
        self.ListLabel = QtWidgets.QLabel(self.FileIOWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ListLabel.sizePolicy().hasHeightForWidth())
        self.ListLabel.setSizePolicy(sizePolicy)
        self.ListLabel.setObjectName("ListLabel")
        self.FileIOLayout.addWidget(self.ListLabel)
        self.FileListWidget = QtWidgets.QListWidget(self.FileIOWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.FileListWidget.sizePolicy().hasHeightForWidth())
        self.FileListWidget.setSizePolicy(sizePolicy)
        self.FileListWidget.setObjectName("FileListWidget")
        self.FileListWidget.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.FileIOLayout.addWidget(self.FileListWidget)
        self.ButtonsWidget = QtWidgets.QWidget(self.FileIOWidget)
        self.ButtonsWidget.setObjectName("ButtonsWidget")
        self.bwlayout = QtWidgets.QHBoxLayout(self.ButtonsWidget)
        self.bwlayout.setObjectName("bwlayout")
        self.AddButton = QtWidgets.QPushButton(self.ButtonsWidget)
        self.AddButton.setObjectName("AddButton")
        self.bwlayout.addWidget(self.AddButton)
        self.RemoveButton = QtWidgets.QPushButton(self.ButtonsWidget)
        self.RemoveButton.setObjectName("RemoveButton")
        self.bwlayout.addWidget(self.RemoveButton)
        self.OutputLabel = QtWidgets.QLabel(self.ButtonsWidget)
        self.OutputLabel.setObjectName("OutputLabel")
        self.bwlayout.addWidget(self.OutputLabel)
        self.OutputLine = QtWidgets.QLineEdit(self.ButtonsWidget)
        self.OutputLine.setObjectName("OutputLine")
        self.bwlayout.addWidget(self.OutputLine)
        self.BrowseButton = QtWidgets.QToolButton(self.ButtonsWidget)
        self.BrowseButton.setObjectName("BrowseButton")
        self.bwlayout.addWidget(self.BrowseButton)
        self.FileIOLayout.addWidget(self.ButtonsWidget)
        self.fiolayout.addLayout(self.FileIOLayout)
        self.MainDialogLayout.addWidget(self.FileIOWidget)
        self.TopLine = QtWidgets.QFrame(MainDialog)
        self.TopLine.setFrameShape(QtWidgets.QFrame.HLine)
        self.TopLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.TopLine.setObjectName("TopLine")
        self.MainDialogLayout.addWidget(self.TopLine)
        self.OpsWidget = QtWidgets.QWidget(MainDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.OpsWidget.sizePolicy().hasHeightForWidth())
        self.OpsWidget.setSizePolicy(sizePolicy)
        self.OpsWidget.setObjectName("OpsWidget")
        self.owlayout = QtWidgets.QHBoxLayout(self.OpsWidget)
        self.owlayout.setContentsMargins(0, 0, 0, 0)
        self.owlayout.setObjectName("owlayout")
        self.XMLWidget = QtWidgets.QWidget(self.OpsWidget)
        self.XMLWidget.setObjectName("XMLWidget")
        self.xwlayout = QtWidgets.QVBoxLayout(self.XMLWidget)
        self.xwlayout.setContentsMargins(0, 0, 0, 0)
        self.xwlayout.setSpacing(1)
        self.xwlayout.setObjectName("xwlayout")
        self.XMLLabel = QtWidgets.QLabel(self.XMLWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.XMLLabel.sizePolicy().hasHeightForWidth())
        self.XMLLabel.setSizePolicy(sizePolicy)
        self.XMLLabel.setObjectName("XMLLabel")
        self.xwlayout.addWidget(self.XMLLabel)
        self.XMLLayout = QtWidgets.QVBoxLayout()
        self.XMLLayout.setSpacing(1)
        self.XMLLayout.setObjectName("XMLLayout")
        self.RbManyToMany = QtWidgets.QRadioButton(self.XMLWidget)
        self.RbManyToMany.setObjectName("RbManyToMany")
        self.XMLLayout.addWidget(self.RbManyToMany)
        self.RbManyToOne = QtWidgets.QRadioButton(self.XMLWidget)
        self.RbManyToOne.setObjectName("RbManyToOne")
        self.XMLLayout.addWidget(self.RbManyToOne)
        self.xwlayout.addLayout(self.XMLLayout)
        self.owlayout.addWidget(self.XMLWidget)
        self.LeftLine = QtWidgets.QFrame(self.OpsWidget)
        self.LeftLine.setFrameShape(QtWidgets.QFrame.VLine)
        self.LeftLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.LeftLine.setObjectName("LeftLine")
        self.owlayout.addWidget(self.LeftLine)
        self.DimWidget = QtWidgets.QWidget(self.OpsWidget)
        self.DimWidget.setObjectName("DimWidget")
        self.dwlayout = QtWidgets.QVBoxLayout(self.DimWidget)
        self.dwlayout.setContentsMargins(0, 0, 0, 0)
        self.dwlayout.setSpacing(1)
        self.dwlayout.setObjectName("dwlayout")
        self.DimLabel = QtWidgets.QLabel(self.DimWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.DimLabel.sizePolicy().hasHeightForWidth())
        self.DimLabel.setSizePolicy(sizePolicy)
        self.DimLabel.setObjectName("DimLabel")
        self.dwlayout.addWidget(self.DimLabel)
        self.DimWidgetLayout = QtWidgets.QVBoxLayout()
        self.DimWidgetLayout.setObjectName("DimWidgetLayout")
        self.Rb2D = QtWidgets.QRadioButton(self.DimWidget)
        self.Rb2D.setObjectName("Rb2D")
        self.DimWidgetLayout.addWidget(self.Rb2D)
        self.Rb3D = QtWidgets.QRadioButton(self.DimWidget)
        self.Rb3D.setObjectName("Rb3D")
        self.DimWidgetLayout.addWidget(self.Rb3D)
        self.dwlayout.addLayout(self.DimWidgetLayout)
        self.owlayout.addWidget(self.DimWidget)
        self.MiddleLine = QtWidgets.QFrame(self.OpsWidget)
        self.MiddleLine.setFrameShape(QtWidgets.QFrame.VLine)
        self.MiddleLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.MiddleLine.setObjectName("MiddleLine")
        self.owlayout.addWidget(self.MiddleLine)
        self.ContentWidget = QtWidgets.QWidget(self.OpsWidget)
        self.ContentWidget.setObjectName("ContentWidget")
        self.cwlayout = QtWidgets.QVBoxLayout(self.ContentWidget)
        self.cwlayout.setContentsMargins(0, 0, 0, 0)
        self.cwlayout.setSpacing(1)
        self.cwlayout.setObjectName("cwlayout")
        self.ContentsLabel = QtWidgets.QLabel(self.ContentWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ContentsLabel.sizePolicy().hasHeightForWidth())
        self.ContentsLabel.setSizePolicy(sizePolicy)
        self.ContentsLabel.setObjectName("ContentsLabel")
        self.cwlayout.addWidget(self.ContentsLabel)
        self.ContentWidgetLayout = QtWidgets.QVBoxLayout()
        self.ContentWidgetLayout.setObjectName("ContentWidgetLayout")
        self.RbMarkers = QtWidgets.QRadioButton(self.ContentWidget)
        self.RbMarkers.setObjectName("RbMarkers")
        self.ContentWidgetLayout.addWidget(self.RbMarkers)
        self.RbROIs = QtWidgets.QRadioButton(self.ContentWidget)
        self.RbROIs.setObjectName("RbROIs")
        self.ContentWidgetLayout.addWidget(self.RbROIs)
        self.RbMarkersAndROIs = QtWidgets.QRadioButton(self.ContentWidget)
        self.RbMarkersAndROIs.setObjectName("RbMarkersAndROIs")
        self.ContentWidgetLayout.addWidget(self.RbMarkersAndROIs)
        self.cwlayout.addLayout(self.ContentWidgetLayout)
        self.owlayout.addWidget(self.ContentWidget)
        self.RightLine = QtWidgets.QFrame(self.OpsWidget)
        self.RightLine.setFrameShape(QtWidgets.QFrame.VLine)
        self.RightLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.RightLine.setObjectName("RightLine")
        self.owlayout.addWidget(self.RightLine)
        self.ScalingWidget = QtWidgets.QWidget(self.OpsWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ScalingWidget.sizePolicy().hasHeightForWidth())
        self.ScalingWidget.setSizePolicy(sizePolicy)
        self.ScalingWidget.setObjectName("ScalingWidget")
        self.swlayout = QtWidgets.QVBoxLayout(self.ScalingWidget)
        self.swlayout.setContentsMargins(0, 0, 0, 0)
        self.swlayout.setSpacing(1)
        self.swlayout.setObjectName("swlayout")
        self.ScalingLabel = QtWidgets.QLabel(self.ScalingWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ScalingLabel.sizePolicy().hasHeightForWidth())
        self.ScalingLabel.setSizePolicy(sizePolicy)
        self.ScalingLabel.setObjectName("ScalingLabel")
        self.swlayout.addWidget(self.ScalingLabel)
        self.ScalingLayout = QtWidgets.QFormLayout()
        self.ScalingLayout.setHorizontalSpacing(1)
        self.ScalingLayout.setObjectName("ScalingLayout")
        self.XLabel = QtWidgets.QLabel(self.ScalingWidget)
        self.XLabel.setObjectName("XLabel")
        self.ScalingLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.XLabel)
        self.YLabel = QtWidgets.QLabel(self.ScalingWidget)
        self.YLabel.setObjectName("YLabel")
        self.ScalingLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.YLabel)
        self.ZLabel = QtWidgets.QLabel(self.ScalingWidget)
        self.ZLabel.setObjectName("ZLabel")
        self.ScalingLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.ZLabel)
        self.XLine = QtWidgets.QLineEdit(self.ScalingWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.XLine.sizePolicy().hasHeightForWidth())
        self.XLine.setSizePolicy(sizePolicy)
        self.XLine.setMaximumSize(QtCore.QSize(80, 16777215))
        self.XLine.setObjectName("XLine")
        self.ScalingLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.XLine)
        self.YLine = QtWidgets.QLineEdit(self.ScalingWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.YLine.sizePolicy().hasHeightForWidth())
        self.YLine.setSizePolicy(sizePolicy)
        self.YLine.setMaximumSize(QtCore.QSize(80, 16777215))
        self.YLine.setObjectName("YLine")
        self.ScalingLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.YLine)
        self.ZLine = QtWidgets.QLineEdit(self.ScalingWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ZLine.sizePolicy().hasHeightForWidth())
        self.ZLine.setSizePolicy(sizePolicy)
        self.ZLine.setMaximumSize(QtCore.QSize(80, 16777215))
        self.ZLine.setObjectName("ZLine")
        self.ScalingLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.ZLine)
        self.swlayout.addLayout(self.ScalingLayout)
        self.owlayout.addWidget(self.ScalingWidget)
        self.MainDialogLayout.addWidget(self.OpsWidget)
        self.BottomLine = QtWidgets.QFrame(MainDialog)
        self.BottomLine.setFrameShape(QtWidgets.QFrame.HLine)
        self.BottomLine.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.BottomLine.setObjectName("BottomLine")
        self.MainDialogLayout.addWidget(self.BottomLine)
        self.ConvertButton = QtWidgets.QPushButton(MainDialog)
        self.ConvertButton.setObjectName("ConvertButton")
        self.MainDialogLayout.addWidget(self.ConvertButton)
        self.mdlayout.addLayout(self.MainDialogLayout)

        self.retranslateUi(MainDialog)
        QtCore.QMetaObject.connectSlotsByName(MainDialog)
        self.makeConnections()

    def retranslateUi(self, MainDialog):
        _translate = QtCore.QCoreApplication.translate
        MainDialog.setWindowTitle(_translate("MainDialog", "FIJI Data Converter"))
        self.ListLabel.setText(_translate("MainDialog", "Input Files:"))
        self.AddButton.setText(_translate("MainDialog", "Add..."))
        self.RemoveButton.setText(_translate("MainDialog", "Remove"))
        self.OutputLabel.setText(_translate("MainDialog", "Output Location:"))
        self.BrowseButton.setText(_translate("MainDialog", "..."))
        self.XMLLabel.setText(_translate("MainDialog", "XML Collation:"))
        self.RbManyToMany.setText(_translate("MainDialog", "Many to Many"))
        self.RbManyToOne.setText(_translate("MainDialog", "Many to One"))
        self.DimLabel.setText(_translate("MainDialog", "Dimensionality:"))
        self.Rb2D.setText(_translate("MainDialog", "2D"))
        self.Rb3D.setText(_translate("MainDialog", "3D"))
        self.ContentsLabel.setText(_translate("MainDialog", "Contents:"))
        self.RbMarkers.setText(_translate("MainDialog", "Markers"))
        self.RbROIs.setText(_translate("MainDialog", "Regions"))
        self.RbMarkersAndROIs.setText(_translate("MainDialog", "Markers and Regions"))
        self.ScalingLabel.setText(_translate("MainDialog", "Scaling (\u03BCm/px):"))
        self.XLabel.setText(_translate("MainDialog", "X:"))
        self.YLabel.setText(_translate("MainDialog", "Y:"))
        self.ZLabel.setText(_translate("MainDialog", "Z:"))
        self.ConvertButton.setText(_translate("MainDialog", "Convert"))

	#adds items to the FileListWidget
    def on_AddButton_clicked(self):
        #make a file dialog
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.ExistingFiles)
        filters = list()
        filters.append("Marker files (*.xml)")
        filters.append("Exported ROI data (*.txt)")
        dlg.setNameFilters(filters)
        
        if dlg.exec_():
            #get unique list of selected files
            selected = list()
            selected = dlg.selectedFiles()
            selected = list(set(selected))
            #get list of files already in list widget
            existing = list()
            for ind in range(self.FileListWidget.count()):
                existing.append(self.FileListWidget.item(ind).text())
            #remove files already in list widget from list of selected files
            for item in existing:
                if item in selected:
                    selected.remove(item)
            #add new items
            self.FileListWidget.addItems(selected)
    
    #removes an item from the FileListWidget
    def on_RemoveButton_clicked(self):
        removeThese = self.FileListWidget.selectedItems()
        for item in removeThese:
            self.FileListWidget.takeItem(self.FileListWidget.row(item))
    
    #opens a file explorer dialog to let the user select a folder
    def on_BrowseButton_clicked(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.DirectoryOnly)
        filenames = list()
        
        if dlg.exec_():
            filenames = dlg.selectedFiles()
            self.OutputLine.setText(filenames[0])
    
    #disables ZLine when checked
    def on_Rb2D_selected(self, selected):
        if selected:
            palette = QPalette()
            palette.setColor(QPalette.Base, Qt.gray)
            palette.setColor(QPalette.Text, Qt.darkGray)
            self.ZLine.setPalette(palette)
            self.ZLine.setReadOnly(True)
    
    #enables ZLine when checked
    def on_Rb3D_selected(self, selected):
        if selected:
            palette = QPalette()
            palette.setColor(QPalette.Base, Qt.white)
            palette.setColor(QPalette.Text, Qt.black)
            self.ZLine.setPalette(palette)
            self.ZLine.setReadOnly(False)
    
    #performs conversion
    def on_ConvertButton_clicked(self):
        #check all inputs are given
        chkr = self.checkSetup()
        opaths = list()
        if not chkr.check():
            #MessageBox to inform user of missing inputs
            msg = "Please choose settings for:\r\n\r\n"
            msg += chkr.getErrors()
            errMsg = QMessageBox()
            errMsg.setIcon(QMessageBox.Critical)
            errMsg.setText(msg)
            errMsg.setWindowTitle("Invalid Settings")
            errMsg.exec_()
            
        else:
            #put data from inputs into converter object
            files = list()
            for ind in range(self.FileListWidget.count()):
                files.append(self.FileListWidget.item(ind).text())
            convertops = Converter()
            convertops.files = files
            convertops.output = self.OutputLine.text()
            convertops.doManyXML = self.RbManyToMany.isChecked()
            convertops.do3D = self.Rb3D.isChecked()
            if self.RbMarkersAndROIs.isChecked():
                convertops.doMarkers = True
                convertops.doROIs = True
            elif self.RbMarkers.isChecked():
                convertops.doMarkers = True
            else:
                convertops.doROIs = True
            convertops.scaleX = float(self.XLine.text())
            convertops.scaleY = float(self.YLine.text())
            if convertops.do3D:
                convertops.ZSpacing = float(self.ZLine.text())
            #do the conversion
            self.ConvertButton.setText("Converting data...")
            self.dialogDisabled(True)
            data = convertops.convert()
            for datum in data:
                opaths.append(datum[0])
                if path.exists(datum[0]):
                    overwriteBox = QMessageBox()
                    overwriteBox.setIcon(QMessageBox.Warning)
                    overwriteBox.setText(datum[0] + " already exists. Overwrite?")
                    overwriteBox.setWindowTitle("File Already Exists")
                    overwriteBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
                    if overwriteBox.exec_() == QMessageBox.Yes:
                        convertops.writeFile(datum[0], datum[1])
                else:
                    convertops.writeFile(datum[0], datum[1])
            doneBox = QMessageBox()
            doneBox.setIcon(QMessageBox.Information)
            doneBox.setText("Conversion complete")
            doneBox.setStandardButtons(QMessageBox.Ok)
            doneBox.exec_()
            ui = Ui_mdgForm()
            tacmsgbox = QtWidgets.QDialog()
            ui.setupUi(tacmsgbox)
            ui.addfromlist(opaths)
            _ = tacmsgbox.exec_()
            self.FileListWidget.clear()
            self.OutputLine.clear()
            self.ConvertButton.setText("Convert")
            self.dialogDisabled(False)

    
    def makeConnections(self):
        self.AddButton.clicked.connect(self.on_AddButton_clicked)
        self.RemoveButton.clicked.connect(self.on_RemoveButton_clicked)
        self.BrowseButton.clicked.connect(self.on_BrowseButton_clicked)
        self.Rb2D.toggled.connect(self.on_Rb2D_selected)
        self.Rb3D.toggled.connect(self.on_Rb3D_selected)
        self.ConvertButton.clicked.connect(self.on_ConvertButton_clicked)
    
    def checkSetup(self):
        #make object to hold check info
        chkr = SetupCheckErr()
        #must have at least one input file
        if self.FileListWidget.count() >= 1:
            chkr.inputs = True
        #must have an output location
        if len(self.OutputLine.text()) >= 1:
            chkr.output = True
        #must have XML option selected
        xmlBool = (
            self.RbManyToMany.isChecked()
            or self.RbManyToOne.isChecked()
        )
        if xmlBool:
            chkr.XMLChecked = True
        #must have dimension selected
        dimBool = (
            self.Rb2D.isChecked()
            or self.Rb3D.isChecked()
        )
        if dimBool:
            chkr.dimChecked = True
        #must have contents selected
        contentBool = (
            self.RbMarkers.isChecked()
            or self.RbROIs.isChecked()
            or self.RbMarkersAndROIs.isChecked()
        )
        if contentBool:
            chkr.contentsChecked = True
        #must have scaling entered
        scalingBool = (
            len(self.XLine.text()) > 0
            and len(self.YLine.text()) > 0
        )
        if self.Rb3D.isChecked():
            scalingBool = scalingBool and len(self.ZLine.text()) > 0
        if scalingBool:
            chkr.scaling = True
        return chkr

    def dialogDisabled(self, disabled):
        self.FileIOWidget.setDisabled(disabled)
        self.OpsWidget.setDisabled(disabled)
        self.ConvertButton.setDisabled(disabled)



from . reader import ReaderMixin
from . writer import WriterMixin


class Converter(ReaderMixin, WriterMixin):
    
    markers = {
        1: "Dot", 
        2: "OpenCircle", 
        3: "Cross", 
        4: "Plus", 
        5: "OpenUpTriangle", 
        6: "OpenDownTriangle", 
        7: "OpenSquare", 
        8: "Asterisk", 
        9: "OpenDiamond", 
        10: "FilledStar", 
        11: "FilledCircle", 
        12: "FilledUpTriangle", 
        13: "FilledDownTriangle", 
        14: "FilledSquare", 
        15: "FilledDiamond", 
        16: "Flower", 
        17: "OpenStar", 
        18: "DoubleCircle", 
        19: "Circle1", 
        20: "Circle2", 
        21: "Circle3", 
        22: "Circle4", 
        23: "Circle5", 
        24: "Circle6", 
        25: "Circle7", 
        26: "Circle8", 
        27: "Circle9", 
        28: "Flower2", 
        29: "SnowFlake", 
        30: "OpenFinial", 
        31: "FilledFinial", 
        32: "MalteseCross", 
        33: "FilledQuadStar", 
        34: "OpenQuadStar", 
        35: "Flower3", 
        36: "Pinwheel", 
        37: "TexacoStar", 
        38: "ShadedStar", 
        39: "SkiBasket", 
        40: "Clock", 
        41: "ThinArrow", 
        42: "ThickArrow", 
        43: "SquareGunSight", 
        44: "GunSight", 
        45: "TriStar", 
        46: "NinjaStar", 
        47: "KnightsCross", 
        48: "Splat", 
        49: "CircleArrow", 
        50: "CircleCross"
    }
    
    boilerplateXML = {
        'header': ('<?xml version="1.0" encoding="ISO-8859-1"?>\r\n'
                         '<mbf version="4.0" xmlns="http://www.mbfbioscience.com/2007/neurolucida"'
                         ' xmlns:nl="http://www.mbfbioscience.com/2007/neurolucida" appname="Neurolucida"'
                         ' appversion="2018.2.2 (64-bit)">\r\n<description><![CDATA[]]></description>\r\n'
                        ), 
        'footer': '</mbf>\r\n', 
        'contour props': ('  <property name="GUID"><s></s></property>\r\n'
                                     '  <property name="FillDensity"><n>0</n></property>\r\n'
                                     '  <resolution>1.000000</resolution>\r\n'
                                    )
    }
    
    def __init__(self):
        self.files = list()
        self.markerFiles = list()
        self.roiFiles = list()
        self.output = str()
        self.doManyXML = False
        self.do3D = False
        self.doMarkers = False
        self.doROIs = False
        self.scaleX = 1.0
        self.scaleY = 1.0
        self.ZSpacing = 0.0
    
    def propStr(self):
        #holds string
        retstr = str()
        #go through bools
        if self.doManyXML:
            retstr += "XML:\t\t\tto Many\r\n"
        else:
            retstr += "XML:\t\t\tto One\r\n"
        if self.do3D:
            retstr += "Dimensionality:\t\t3D\r\n"
        else:
            retstr += "Dimensionality:\t\t2D\r\n"
        if self.doMarkers:
            retstr += "Markers:\t\t\tYes\r\n"
        else:
            retstr += "Markers:\t\t\tNo\r\n"
        if self.doROIs:
            retstr += "Regions:\t\t\tYes\r\n"
        else:
            retstr += "Regions:\t\t\tNo\r\n"
        #whitespace
        retstr += "\r\n"
        #go through scaling data
        retstr += "X: " + str(self.scaleX) + "\r\n"
        retstr += "Y: " + str(self.scaleY) + "\r\n"
        if self.do3D:
            retstr += "Z: " + str(self.ZSpacing) + "\r\n"
        else:
            retstr += "Z: " + "N/A\r\n"
        #whitespace
        retstr += "\r\n"
        #output path
        retstr += "Output: " + self.output + "\r\n"
        #whitespace
        retstr += "\r\n"
        #input files
        retstr += "Inputs:\r\n"
        for file in self.files:
            retstr += file + "\r\n"
        return retstr

    def _convertMarkers_toMany(self):
        convertedData = list()
        for ind in range(len(self.markerFiles)):
            image = self.readXMLImage(self.markerFiles[ind])
            data = self.readXMLMarkers(self.markerFiles[ind])
            for markerType in list(data.keys()):
                for markerPoint in range(len(data[markerType])):
                    newPoint = (data[markerType][markerPoint][0] * self.scaleX,
                                       data[markerType][markerPoint][1] * self.scaleY * -1.0, 
                                       data[markerType][markerPoint][2] * self.ZSpacing * -1.0)
                    data[markerType][markerPoint] = newPoint
            filefactsStr = self.xmlFilefactsBlock(1)
            imageStr = self.xmlImageBlock([image], (self.scaleX, self.scaleY))
            markerStr = ""
            for markerType in list(data.keys()):
                markerStr += self.xmlMarkerBlock(markerType, data[markerType], self.markers)
            filenameStr = self.output + "/" + self.markerFiles[ind].strip(".xml").rsplit("/", 1)[1] + "_markers.xml"
            bodyStr = self.boilerplateXML["header"] + filefactsStr + imageStr + markerStr + self.boilerplateXML["footer"]
            convertedData.append((filenameStr, bodyStr))
        return convertedData
            
    def _convertMarkers_toOne(self):
        images = list()
        data = dict()
        for ind in range(len(self.markerFiles)):
            images.append(self.readXMLImage(self.markerFiles[ind]))
            data = self.readXMLMarkers(self.markerFiles[ind], data)
            for markerType in list(data.keys()):
                for markerPoint in range(len(data[markerType])):
                    newPoint = (data[markerType][markerPoint][0] * self.scaleX,
                                       data[markerType][markerPoint][1] * self.scaleY * -1.0, 
                                       data[markerType][markerPoint][2] * self.ZSpacing * -1.0)
                    data[markerType][markerPoint] = newPoint                      
        filefactsStr = self.xmlFilefactsBlock(1)
        imageStr = self.xmlImageBlock(images, (self.scaleX, self.scaleY))
        markerStr = ""
        for markerType in list(data.keys()):
                markerStr += self.xmlMarkerBlock(markerType, data[markerType], self.markers)
        filenameStr = self.output + "/" + self.markerFiles[0].strip(".xml").rsplit("/", 1)[1] + "_markers.xml"
        bodyStr = self.boilerplateXML["header"] + filefactsStr + imageStr + markerStr + self.boilerplateXML["footer"]
        return (filenameStr, bodyStr)
    
    def _convertROIs_toMany(self):
        convertedData = list()
        for ind in range(len(self.roiFiles)):
            roi = self.readROI(self.roiFiles[ind])
            for coord in range(len(roi)):
                newPoint = (roi[coord][0] * self.scaleX, 
                                   roi[coord][1] * self.scaleY * -1.0)
                roi[coord] = newPoint
                if self.do3D:
                    roi[coord] = roi[coord] + (ind * self.ZSpacing, ind + 1)
            filenameStr = self.output + "/" + self.roiFiles[ind].strip(".txt").rsplit("/", 1)[1] + "_rois.xml"
            if self.do3D:
                filefactsStr = self.xmlFilefactsBlock(len(self.roiFiles), self.ZSpacing)
            else:
                filefactsStr = self.xmlFilefactsBlock(1)
            roiStr = self.xmlContourBlock(ind + 1, roi, self.boilerplateXML["contour props"])
            bodyStr = self.boilerplateXML["header"] + filefactsStr + roiStr + self.boilerplateXML["footer"]
            convertedData.append((filenameStr, bodyStr))
        return convertedData
    
    def _convertROIs_toOne(self):
        filenameStr = self.output + "/" + self.roiFiles[0].strip(".txt").rsplit("/", 1)[1] + "_rois.xml"
        if self.do3D:
            filefactsStr = self.xmlFilefactsBlock(len(self.roiFiles), self.ZSpacing)
        else:
            filefactsStr = self.xmlFilefactsBlock(1)
        roiStr = ""
        for ind in range(len(self.roiFiles)):
            roi = self.readROI(self.roiFiles[ind])
            for coord in range(len(roi)):
                newPoint = (roi[coord][0] * self.scaleX, 
                                   roi[coord][1] * self.scaleY * -1.0)
                roi[coord] = newPoint
                if self.do3D:
                    roi[coord] = roi[coord] + (ind * self.ZSpacing, ind + 1)
            roiStr += self.xmlContourBlock(ind + 1, roi, self.boilerplateXML["contour props"])
        bodyStr = self.boilerplateXML["header"] + filefactsStr + roiStr + self.boilerplateXML["footer"]
        return (filenameStr, bodyStr)

    def convert(self, saveSettings = False):
        if saveSettings:
            with open(self.output + "\\settings.txt", "w") as file:
                file.write(self.propStr())
        data = list()
        for file in self.files:
            if ".xml" in file:
                self.markerFiles.append(file)
            elif ".txt" in file:
                self.roiFiles.append(file)
        if self.doMarkers:
            if self.doManyXML:
                data.extend(self._convertMarkers_toMany())
            else:
                data.append(self._convertMarkers_toOne())
        if self.doROIs:
            if self.doManyXML:
                data.extend(self._convertROIs_toMany())
            else:
                data.append(self._convertROIs_toOne())
        return data
